import * as THREE from 'three';
import {
    REVISION
} from 'three';
import {
    OrbitControls
} from 'three/examples/jsm/controls/OrbitControls'
import Stats from 'stats.js'
import {
    Pane
} from 'tweakpane'


let debug = true;

export default class Sketch {
    constructor() {
        if (debug) console.log("Three Version: ", REVISION);
        // 初始化
        this.initParam();
        this.init();
        // 调试
        this.initAxesHelp();
        this.initGridHelp();
        this.initController();
        this.openStatus();
        this.initPanel();
        this.render();
        // 绘制一个圆形
        this.generateCircular();
        this.loop();
    }
    // fps监听
    openStatus() {
        this.stats = new Stats();
        this.stats.showPanel(0); // 0: fps, 1: ms, 2: mb, 3+: custom
        document.body.appendChild(this.stats.dom);
    }
    // 调试面板
    initPanel() {
        const pane = new Pane({
            container: document.querySelector('#debug'),
            title: '首页',
        })
        const first = pane.addFolder({
            title: '参数',
            expanded: true
        })

    }

    /// 环形本体
    generateCircular() {
        let geometry = new THREE.SphereBufferGeometry(1, 30, 30);
        let material = new THREE.MeshBasicMaterial({
            wireframe: true,
            color: 0x00ff00,
            side: THREE.DoubleSide,
        });
        this.circleMesh = new THREE.Mesh(geometry, material);
        this.scene.add(this.circleMesh);

        let num = 7;
        let curvePoints = [];
        for (let i = 0; i < num; i++) {
            let theta = i / num * Math.PI * 2;
            curvePoints.push(new THREE.Vector3().setFromSphericalCoords(
                1, Math.PI / 2 + Math.random() - 0.5, theta,
            ))
        }
        const curve = new THREE.CatmullRomCurve3(curvePoints);
        curve.tension = 0.1;
        curve.clone = true;
        const points = curve.getPoints(50);
        const points_geometry = new THREE.BufferGeometry().setFromPoints(points);
        const ponts_material = new THREE.LineBasicMaterial({
            color: 'blue',
            side: THREE.DoubleSide,
        });
        const points_mesh = new THREE.Mesh(points_geometry, ponts_material);
        this.scene.add(points_mesh);
    }

    loop() {
        if (this.stats) {
            this.stats.begin();
            this.stats.end();
        }
        const Delta = this.clock.getDelta();
        this.index = window.requestAnimationFrame(this.loop.bind(this));
        // this.circleGroup.rotation.z += -Delta * 0.1;
        this.render();
    }
    initParam() {
        this.width = window.innerWidth;
        this.height = window.innerHeight;
        this.aspect = this.width / this.height;
        this.clock = new THREE.Clock();
    }
    init() {
        const {
            innerWidth,
            innerHeight
        } = window;
        this.aspect = innerWidth / innerHeight;
        // 场景
        this.scene = new THREE.Scene();
        this.scene.background = new THREE.Color('#eeeeee');
        // 摄像机
        this.camera = new THREE.PerspectiveCamera(75, this.aspect, 0.1, 1000);
        this.camera.position.set(0, 1, 3);
        this.camera.lookAt(0, 0, 0);
        // 渲染
        this.aspect = innerWidth / innerHeight;
        this.renderer = new THREE.WebGLRenderer({
            antialias: true,
            alpha: true,
        });
        this.renderer.setSize(innerWidth, innerHeight);
        this.renderer.render(this.scene, this.camera);
        this.renderer.setPixelRatio = window.Pixi
        document.body.appendChild(this.renderer.domElement);
    }
    initAxesHelp() {
        this.axesHelper = new THREE.AxesHelper(100);
        this.scene.add(this.axesHelper);
    }
    initGridHelp() {
        this.gridHelper = new THREE.GridHelper(100, 100);
        this.scene.add(this.gridHelper);
    }
    initController() {
        const that = this;
        this.controller = new OrbitControls(this.camera, this.renderer.domElement);
        this.controller.addEventListener('change', function () {
            that.renderer.render(that.scene, that.camera);
        })
    }
    render() {
        this.renderer.render(this.scene, this.camera);
    }
}